@@ -2,7 +2,7 @@
    
    Geeks Artificial Neural Network (G.A.N.N) - Main File
    
-   Copyright GNU/GPL 2006 George Delaportas (qtgeo)
+   Copyright GNU/GPL 2005-2006 George Delaportas (qtgeo)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,14 +34,13 @@
 #include "grand.cpp"
 #include "gsocket.cpp"
 #include "gneuron.cpp"
-#include "gstat.cpp"
+#include "gerrstat.cpp"
 #include "gerrc.cpp"
 #include "gdmloader.cpp"
 #include "ggendata.cpp"
 #include "greaddata.cpp"
 #include "gsavedata.cpp"
 #include "gremdata.cpp"
-#include "gnmopt.cpp"
 #include "gsnmc.cpp"
 #include "gnms.cpp"
 #include "gumsgs.cpp"
