@@ -2,7 +2,7 @@
    
    Geeks Artificial Neural Network (G.A.N.N) - Generate Random Data
    
-   Copyright GNU/GPL 2006 George Delaportas (qtgeo)
+   Copyright GNU/GPL 2005-2006 George Delaportas (qtgeo)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,16 +22,14 @@
 
 
 
-//Geeks Generate Data (Function)
+//Common Varialbe
+static float RandomData;	//Random Values
+
+//Geeks Generate Data - Value (Function)
 float GGenData(int FuncType,float Val)
 
 {
 
-    float RandomData;
-    
-    //Randomize
-    GRand();
-    
     //Check Function Type (0:Binary,1:Real)
     if (FuncType==0 && Val==0.0)
     
@@ -68,30 +66,25 @@ float GGenData(int FuncType,float Val)
 
 
 
-//Geeks Generate Data (Function) [Over-Loaded]
+//Geeks Generate Data - Value In Range (Function) [Over-Loaded]
 float GGenData(float Val1,float Val2)
 
 {
 
-    float RandomData;
-    
-    //Randomize
-    GRand();
-    
     //Check Range
     if (Val2>=Val1)
     
     {
     
-        //Produce Random Real Data
-        RandomData=Val1+rand()%int(Val2+1);
+        //Produce Random Real Data In Specified Range
+        RandomData=Val2*rand()/(RAND_MAX+1.0);
     
     }
     
     else
     
     {
-
+    
         printf("Error: Wrong Function Type Or Range Specified!\n");
         
         exit(1);
