@@ -2,7 +2,7 @@
    
    Geeks Artificial Neural Network (G.A.N.N) - Neural Map Simulator
    
-   Copyright GNU/GPL 2006 George Delaportas (qtgeo)
+   Copyright GNU/GPL 2005-2006 George Delaportas (qtgeo)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,13 +28,18 @@ void GNMS(int SimFunc)
 {
 
     //Variables
-    register int i,j,k,n,p;	//Counters
+    register int i,j;		//Fast Counters
+    int n,k,p;			//Simple Counters
+    int GNC;			//GNeurons Counter
+    int NLH;			//Next Layer Head
+    int NLE;			//Next Layer End
     int GNFT;			//GNeuron Function Type
-    int Offset;			//An Offset
     int Tracer;			//A Tracer
-    float NData[MAX_PIN];	//New Data From Inputs GSocket
-    float GData[MAX_PIN];	//Data Produced By G.A.N.N
-    float ErrStat;		//Error Statistics (%)
+    float NewData[MAX_PIN];	//New Data From Inputs GSocket
+    float GMinData[MAX_PIN];	//Min Data Value Produced By G.A.N.N
+    float GMaxData[MAX_PIN];	//Max Data Value Produced By G.A.N.N
+    float TErr;			//Threshold Error (%)
+    float GErr;			//G.A.N.N Error (%)
     float Tmp;			//Temp Values
     
     //Check For Wrong Simulation Mode
@@ -56,9 +61,12 @@ void GNMS(int SimFunc)
     Inputs=InSocket.GetPins();
     Outputs=OutSocket.GetPins();
     
+    //Get Threshold Error Statistics
+    TErr=100.0-Threshold;
+    
     //Fill NData With Inputs GSocket Data (Only For Uniform Checks)
     for (i=0;i<Inputs;i++)
-        NData[i]=InSocket.GetPV(i,0);
+        NewData[i]=InSocket.GetPV(i,0);
     
     //Check GMap Simulation Type So As To Set GNeurons Function Type
     if (GMSType==0)
@@ -100,11 +108,20 @@ void GNMS(int SimFunc)
             //Escape Caption
             ReRoute:;
             
+            //Randomize
+            GRand();
+            
             //Reset Tracer
             Tracer=0;
             
-            //Reset Offest
-            Offset=0;
+            //Reset GNC
+            GNC=-1;
+            
+            //Reset NLH
+            NLH=0;
+            
+            //Reset NLE
+            NLE=0;
             
             //Start Routing Through The Layers
             for (i=0;i<Layers;i++)
@@ -120,10 +137,13 @@ void GNMS(int SimFunc)
                 {
                 
                     //Enable All GNeurons In This Layer
-                    for (j=Offset;j<(k+Offset);j++)
+                    for (j=NLE;j<(k+NLE);j++)
                     
                     {
                     
+                        //GNeurons Counter
+                        GNC++;
+                        
                         //Get Data From InSocket (First Layer)
                         if (i==0)
                         
@@ -141,14 +161,14 @@ void GNMS(int SimFunc)
                                     InSocket.GetPV(p,1));
                                 
                                 //Give Data To This GNeuron
-                                GANN[j].InData(p,Tmp);
+                                GANN[GNC].InData(p,Tmp);
                             
                             }
                             
                             //Enable This GNeuron
-                            GANN[j].Enable(GNFT);
+                            GANN[GNC].Enable(GNFT);
                             
-                            printf("Neuron %i Enabled!\n\n",j+1);
+                            printf("Neuron %i Enabled!\n\n",GNC+1);
                         
                         }
                         
@@ -157,27 +177,29 @@ void GNMS(int SimFunc)
                         
                         {
                         
-                            for (p=0;p<Offset;p++)
+                            for (p=NLH;p<(NLH+NLE);p++)
                             
                             {
                             
                                 //Give Data To This GNeuron
-                                GANN[j].InData(p,GANN[p].OutData());
+                                GANN[GNC].InData(p-NLH,GANN[p].OutData());
                             
                             }
                             
                             //Enable This GNeuron
-                            GANN[j].Enable(GNFT);
+                            GANN[GNC].Enable(GNFT);
                             
-                            printf("Neuron %i Enabled!\n\n",j+1);
+                            printf("Neuron %i Enabled!\n\n",GNC+1);
                         
                         }
                     
                     }
                     
-                    //Set New Offset
-                    if (i<Layers-1)
-                        Offset=k;
+                    //Next Layer Head
+                    NLH+=NLE;
+                    
+                    //Next Layer End
+                    NLE=k;
                 
                 }
                 
@@ -196,21 +218,51 @@ void GNMS(int SimFunc)
             j=0;
             
             //Fill GData
-            for (i=Offset;i<(k+Offset);i++)
+            for (i=NLH;i<(NLH+NLE);i++)
             
             {
             
-                GData[j]=GANN[i].OutData();
+                if (OutSocket.GetPV(j,1)!=0.0)
+                
+                {
+                
+                    GMinData[j]=GANN[i].OutData()*OutSocket.GetPV(j,0);
+                    GMaxData[j]=GANN[i].OutData()*OutSocket.GetPV(j,1);
+                
+                }
+                
+                else
+                
+                {
+                
+                    GMinData[j]=GANN[i].OutData()*OutSocket.GetPV(j,0);
+                    GMaxData[j]=0.0;
+                
+                }
                 
                 j++;
             
             }
             
-            //Gather Error Statistics (0:Uniform,1:Non-Uniform)
-            ErrStat=GStat(NData,GData,ECT);
+            //Print GData
+            printf("----- GData -----\n");
+            
+            for (i=0;i<j;i++)
+            
+            {
+            
+                printf("Minimum Data%i:%.1f\n",i+1,GMinData[i]);
+                printf("Maximum Data%i:%.1f\n",i+1,GMaxData[i]);
+            
+            }
+            
+            printf("-----------------\n\n");
+            
+            //Gather Error Statistics (ECT - 0:Uniform,1:Non-Uniform)
+            GErr=GErrStat(NewData,GMinData,GMaxData,ECT);
             
             //Check Error Statistics
-            if (ErrStat>=Threshold)
+            if (GErr<=TErr)
             
             {
             
@@ -225,7 +277,8 @@ void GNMS(int SimFunc)
                 {
                 
                     printf("GPin:%i\n",i+1);
-                    printf("Value:%.1f\n",GData[i]);
+                    printf("Min Value:%.1f\n",GMinData[i]);
+                    printf("Max Value:%.1f\n",GMaxData[i]);
                 
                 }
                 
@@ -239,17 +292,34 @@ void GNMS(int SimFunc)
             
             {
             
+                //Reset j
+                j=0;
+                
                 //Error Correction
                 for (i=0;i<Outputs;i++)
                 
                 {
                 
-                    if (GData[i]<OutSocket.GetPV(i,0))
-                        GErrC(GANN,i+Offset,ErrStat,0);
+                    for (j=0;j<GNNum;j++)
                     
-                    if (OutSocket.GetPV(i,1)!=0.0 &&\
-                    GData[i]>OutSocket.GetPV(i,1))
-                        GErrC(GANN,i+Offset,ErrStat,0);
+                    {
+                    
+                        Tmp=OutSocket.GetPV(i,1);
+                        
+                        if (OutSocket.GetPV(i,1)!=0.0 &&\
+                        (GANN[j].OutData()*Tmp)>OutSocket.GetPV(i,1))
+                            GErrC(GANN,j,GErr,AUTO);
+                        
+                        Tmp=OutSocket.GetPV(i,0);
+                        
+                        if (OutSocket.GetPV(i,1)==0.0 &&\
+                        (GANN[j].OutData()*Tmp)>OutSocket.GetPV(i,0))
+                            GErrC(GANN,j,GErr,AUTO);
+                        
+                        if ((GANN[j].OutData()*Tmp)<OutSocket.GetPV(i,0))
+                            GErrC(GANN,j,GErr,AUTO);
+                    
+                    }
                 
                 }
                 
